@@ -221,7 +221,7 @@ static unsigned int tcp_base_send_hop(lft_session_params * sess, short nhop, int
 	blen = sizeof(struct ip) + packet->lsrr.ipl_len + sizeof(struct tcphdr) + packet->payload_len;
     
     /* Packet is ready, fire away */
-    if (sendto (sess->send_sock, buf, blen, 0, (struct sockaddr *)&dest, sizeof (dest)) < 0) {
+    if (sendto (sess->send_sock, buf, blen, 0, (const struct sockaddr *)(const void *)&dest, sizeof (dest)) < 0) {
         LFTErrHandler(sess, ERR_RAW_TCP_DISABLED, NULL);
         free(pinfo);
         return 0;
@@ -395,7 +395,7 @@ static void tcp_base_recv_packet (lft_session_params * sess, unsigned int seq, s
                 sess->num_hops = tp->hopno;
                 tp->is_done = 1;
 				sess->hop_info[tp->hopno].done_packet = tp;
-                if (sess->noisy > 1 && sess->target_open == -1)
+                if (sess->noisy > 1 && sess->target_open < 1)
                     LFTEvtHandler(sess,EVT_TCP_PORT_CLOSED,NULL);
                 else 
                     if (sess->noisy > 1 && sess->target_open > 0)
@@ -415,14 +415,30 @@ static void tcp_base_finish(lft_session_params * sess)
     int hopno;
     int maxhop;
     int reply, noreply;
-    int noreply_hop_first = 0;
     int as_for_hop = 0;
     struct trace_packet_info_s 	*tp;
     char *netname; 
 	/*int ocres;*/
-    char ind;
     char *myApp = (char *)malloc((strlen(version) * sizeof(char)) + 1 + (strlen(appname) * sizeof(char)));
     struct ip_list_array *ipaslist = (struct ip_list_array *)malloc(sizeof(struct ip_list_array));
+	/* Variables for seam detection */
+	int icmpcode;
+	int asseam_hopno=-1;
+	struct in_addr asseam_hopaddr;
+	int netseam_hopno=-1;
+	struct in_addr netseam_hopaddr;
+	struct in_addr classbmask;
+	struct in_addr masked_target;
+	int prevasn=-1;
+	struct in_addr prevasn_hopaddr;
+	int prevasn_hopno;
+	int lastishole;
+	int netreached=0;
+	int isseam;
+	/* ---------------------------- */
+	inet_aton("255.255.0.0", &classbmask);
+	masked_target.s_addr=sess->remote_address.s_addr & classbmask.s_addr;
+
     EvtPacketInfoParam ehip;
 
     memset(ipaslist, '0', sizeof(struct ip_list_array));
@@ -533,7 +549,85 @@ static void tcp_base_finish(lft_session_params * sess)
 		free(ipaslist);
 		return;
 	}
-
+	
+	/* seam detection */
+	as_for_hop=0;
+	for(hopno = sess->ttl_min; hopno < sess->hop_info_length; hopno++)
+	{
+		struct in_addr last_hop;
+		icmpcode=-100;
+		last_hop.s_addr = 0;
+		if(sess->hop_info[hopno].all_rcvd)
+		{
+			lastishole=0;
+            SLIST_FOREACH(tp, &(sess->hop_info[hopno].packets), next_by_hop)
+			{
+                if(tp->recv.tv_sec)
+				{
+					if(hopno<=maxhop)
+						icmpcode=tp->icmp_type;
+					if(last_hop.s_addr != tp->hopaddr.s_addr)
+					{
+						if((tp->hopaddr.s_addr & classbmask.s_addr) == masked_target.s_addr)
+							netreached=1;
+						else
+						{
+							netseam_hopno=hopno;
+							netseam_hopaddr=tp->hopaddr;
+						}
+						if(sess->do_aslookup || sess->do_netlookup)
+						{
+							if (sess->use_radb)
+							{ 
+								/* using RADB/IRR */
+								tp->asnumber = w_lookup_as(sess->wsess, inet_ntoa(tp->hopaddr));
+							}
+							else
+							{
+								/* using pwhois by default */
+								tp->asnumber = (*ipaslist).asn[as_for_hop];
+							}
+							if(prevasn==-1)
+							{
+								if(tp->asnumber)
+								{
+									prevasn=tp->asnumber;
+									prevasn_hopno=hopno;
+									prevasn_hopaddr=tp->hopaddr;
+								}
+							}
+							else
+							{
+								if(tp->asnumber)
+								{
+									if(tp->asnumber!=prevasn)
+									{
+										asseam_hopno=prevasn_hopno;
+										asseam_hopaddr=prevasn_hopaddr;
+									}
+									prevasn=tp->asnumber;
+									prevasn_hopno=hopno;
+									prevasn_hopaddr=tp->hopaddr;
+								}
+							}
+						}
+						last_hop=tp->hopaddr;
+					}
+				}
+			}
+			as_for_hop++;
+		}
+		else
+			lastishole=1;
+		if(icmpcode==-1)
+			break;
+	}
+	if(!netreached)
+		netseam_hopno=-1;
+	if(lastishole)
+		asseam_hopno=-1;
+	/* -------------- */
+	
 	noreply = 0;
 	reply = 0;
 	as_for_hop = 0;            /* this correlates the hopno to the asn stored in ipaslist */
@@ -554,8 +648,6 @@ static void tcp_base_finish(lft_session_params * sess)
 					return;
 				}
 			}
-			
-			noreply_hop_first = 0;
 		}
     
 		last_hop.s_addr = 0;
@@ -592,15 +684,12 @@ static void tcp_base_finish(lft_session_params * sess)
 				return;
 			}
 
-			ind = ' ';
-        
 			SLIST_FOREACH(tp, &(sess->hop_info[hopno].packets), next_by_hop) {
 				
 				if (tp->recv.tv_sec) {
 					reply = 1;
 																	
 					if (last_hop.s_addr != tp->hopaddr.s_addr) {
-						ind = ' ';
 						ehip.asnumber = 0; 	/* init/clear the ASN */
 						if (sess->do_aslookup) {
 							if (sess->use_radb) { 
@@ -611,6 +700,8 @@ static void tcp_base_finish(lft_session_params * sess)
 								ehip.asnumber = (*ipaslist).asn[as_for_hop];
 							}
 						}
+						tp->asnumber=ehip.asnumber;
+						ehip.netname=NULL;
 						if (sess->do_netlookup) {
 							if (!sess->do_aslookup || (sess->do_aslookup && !sess->use_cymru && !sess->use_radb)) {
 								netname = (*ipaslist).netName[as_for_hop];
@@ -619,12 +710,66 @@ static void tcp_base_finish(lft_session_params * sess)
 							}
 							ehip.netname=netname;
 						}
+						if(ehip.netname)
+							strncpy(tp->netname, ehip.netname, 511);
+						else
+							tp->netname[0]=0;
 					}
 					ehip.last_hop=last_hop;
+					tp->last_hop=ehip.last_hop;
 					last_hop = tp->hopaddr;
 				}
 				ehip.tp=tp;
-				ind = '/';
+				/* seam processing */
+				isseam=0;
+				ehip.is_asseam=0;
+				ehip.is_netseam=0;
+				ehip.is_open=0;
+				ehip.is_filtered=0;
+				ehip.seam_traced=0;
+				if(sess->check_seam && hopno==asseam_hopno && tp->hopaddr.s_addr==asseam_hopaddr.s_addr)
+				{
+					isseam=1;
+					ehip.is_asseam=1;
+				}
+				if(sess->check_seam && hopno==netseam_hopno && tp->hopaddr.s_addr==netseam_hopaddr.s_addr)
+				{
+					isseam=1;
+					ehip.is_netseam=1;
+				}
+				if(isseam)
+				{
+					if(sess->check_seam)
+					{
+						int curroutputstyle=getOutputStyle();
+						char hostname[100];
+						ehip.seam_traced=1;
+						setOutputStyle(2);
+						lft_session_params * subsess=LFTSessionOpen();
+						strncpy(hostname, inet_ntoa(tp->hopaddr),100);
+						subsess->senddevsel = 1;
+						subsess->senddev = sess->pcap_send_dev;
+						subsess->userdevsel = 1;
+						subsess->userdev = sess->pcap_dev;
+						subsess->auto_ports=0;
+						subsess->dport=179;
+						subsess->seq_start=30;
+						subsess->retry_min=1;
+						subsess->retry_max=1;
+						subsess->resolve_names=0;
+						subsess->ahead_limit=1;
+						subsess->break_on_icmp = 0;
+						subsess->is_graphviz_subquery=1;
+						subsess->hostname=hostname;
+						subsess->hostname_lsrr_size = 0;
+						LFTExecute(subsess);
+						ehip.is_open=subsess->target_open;
+						ehip.is_filtered=subsess->target_filtered;
+						LFTSessionClose(subsess);
+						setOutputStyle(curroutputstyle);
+					}
+				}
+				/* --------------- */
 				LFTEvtHandler(sess,EVT_RPT_PACKET_INFO,&ehip);
 				if(sess->exit_state < 0){
 					free(ipaslist);
@@ -663,9 +808,6 @@ static int tcp_base_check_timeouts(lft_session_params * sess)
     int need_reply = 0;
     int no_reply = 0;
     int last_return = 0;
-    unsigned int tseq;
-    
-    tseq = 0;
     
     gettimeofday (&(sess->now), NULL);
     if (timediff_ms (sess->ts_last_sent, sess->now) < sess->scatter_ms)
@@ -888,7 +1030,7 @@ static void tcp_base_process_packet(lft_session_params * sess, const u_char *pac
 				if ((tcp->th_flags & TH_ACK) && (tcp->th_flags & TH_SYN) && !(tcp->th_flags & TH_RST)) 
 					sess->target_open++;
 				if ((tcp->th_flags & TH_ACK) && !(tcp->th_flags & TH_SYN) && (tcp->th_flags & TH_RST))
-					sess->target_open = -1;
+					sess->target_open = 0;
 			}
                     
             tcp_base_recv_packet(sess, ntohl (tcp->th_ack) - 1, ip->ip_src, -1, hdr);
@@ -933,7 +1075,7 @@ void win_tcp_base_process(lft_session_params * sess)
 #else
 static void pcap_tcp_base_process_packet(u_char * user_data, const struct pcap_pkthdr *hdr, const u_char * packet)
 {
-    lft_session_params * sess=(lft_session_params *)user_data;
+    lft_session_params * sess=(lft_session_params *)(void *)user_data;
 	if(sess->exit_state<0)
 		return;
     tcp_base_process_packet(sess, packet, hdr);
