@@ -96,7 +96,7 @@ int generateICMPPacket(
 	memset(packet->packet, 0,packet->packet_len);
 	//initialize ip header
 	currptr = packet->packet;
-	packet->ip_hdr = (struct ip *)packet->packet;
+	packet->ip_hdr = (struct ip *)(void *)packet->packet;
 	packet->ip_hdr->ip_v = 4;
 	packet->ip_hdr->ip_hl = 5;
 	if(sess->hostname_lsrr_size)
@@ -136,7 +136,7 @@ int generateICMPPacket(
 	*/
 	if(sess->protocol == 3)
 	{
-		packet->icmp_trace_opt = (struct rfc1393_ip_option_s *)currptr;
+		packet->icmp_trace_opt = (struct rfc1393_ip_option_s *)(void *)currptr;
 		packet->icmp_trace_opt->optcode = 82;
 		packet->icmp_trace_opt->optlength = 12;
 		packet->icmp_trace_opt->id = seq;
@@ -158,7 +158,7 @@ int generateICMPPacket(
 	*/
 	if(sess->hostname_lsrr_size)
 	{
-		packet->lsrr=(struct ip_lsrr *)currptr;
+		packet->lsrr=(struct ip_lsrr *)(void *)currptr;
 		currptr+=(sess->hostname_lsrr_size + 1)*4;
 		for(i = 0; i < sess->hostname_lsrr_size; i++)
 			packet->lsrr->data[i] = get_address(sess, sess->hostname_lsrr[i]);
@@ -180,7 +180,7 @@ int generateICMPPacket(
 		u_short sequence;
 	};
 	*/
-	packet->echo=(struct icmp_echo_header_s *)currptr;
+	packet->echo=(struct icmp_echo_header_s *)(void *)currptr;
 	currptr+=sizeof(struct icmp_echo_header_s);
 	packet->echo->type=ICMP_ECHO;
 	packet->echo->code=0;
@@ -196,14 +196,13 @@ int generateICMPPacket(
 				j=0;
 			currptr[i]=payload_fill[j];
 		}
-		currptr+=userlen;
 	}
 	else
 		packet->payload=NULL;
 	packet->payload_len = userlen;
     	sess->payload = NULL;
     	sess->payloadlen = userlen;
-	packet->echo->checksum=in_cksum((u_short *)packet->echo, sizeof(struct icmp_echo_header_s)+packet->payload_len);
+	packet->echo->checksum=in_cksum((u_short *)(void *)packet->echo, sizeof(struct icmp_echo_header_s)+packet->payload_len);
 /*#ifndef SCREWED_IP_LEN
 	packet->echo->checksum = htons(packet->echo->checksum);
 #endif*/
@@ -413,7 +412,7 @@ unsigned int icmp_rfc_send_request(lft_session_params * sess, LFT_CALLBACK err,
 	rfc_unique_seq = nseq;
 	generateICMPPacket(sess, err, evt, &pinfo->u.icmp_packet, 0, sess->icmp_packet.echo->id, rfc_unique_seq);
     	/* Packet is ready, fire away */
-    	if(sendto(sess->send_sock, pinfo->u.icmp_packet.packet, pinfo->u.icmp_packet.packet_len, 0, (const struct sockaddr *)&dest, sizeof (dest)) < 0)
+    	if(sendto(sess->send_sock, pinfo->u.icmp_packet.packet, pinfo->u.icmp_packet.packet_len, 0, (const struct sockaddr *)(const void *)&dest, sizeof (dest)) < 0)
 	{
         	LFTErrHandler(sess, ERR_RAW_TCP_DISABLED, NULL);
         	free(pinfo);
@@ -471,7 +470,7 @@ unsigned int icmp_base_send_hop(lft_session_params * sess, LFT_CALLBACK err, LFT
     	}
 	generateICMPPacket(sess, err, evt, &pinfo->u.icmp_packet, nhop+1, sess->icmp_packet.echo->id, base_unique_seq);
     	/* Packet is ready, fire away */
-    	if(sendto(sess->send_sock, pinfo->u.icmp_packet.packet, pinfo->u.icmp_packet.packet_len, 0, (const struct sockaddr *)&dest, sizeof (dest)) < 0)
+    	if(sendto(sess->send_sock, pinfo->u.icmp_packet.packet, pinfo->u.icmp_packet.packet_len, 0, (const struct sockaddr *)(const void *)&dest, sizeof (dest)) < 0)
 	{
 		/* printf("errno=%d\n",errno); */
         	LFTErrHandler(sess, ERR_RAW_TCP_DISABLED, NULL);
@@ -626,7 +625,7 @@ static void icmp_process_packet(lft_session_params * sess, LFT_CALLBACK err, LFT
 {
     	const struct ip *ip, *orig_ip;
     	const struct icmp *icmp;
-	const struct icmp_echo_header_s *orig_echo, *resp_echo, *tmp_echo;
+	const struct icmp_echo_header_s *orig_echo, *resp_echo;
 
 	LFTErrHandler=err;
 	LFTEvtHandler=evt;
@@ -648,7 +647,6 @@ static void icmp_process_packet(lft_session_params * sess, LFT_CALLBACK err, LFT
 	case IPPROTO_ICMP:
 		orig_ip = ip;
 		icmp = (const void *) packet;
-		tmp_echo=(const struct icmp_echo_header_s *)(packet+14);
 		if(icmp->icmp_type != ICMP_ECHOREPLY && icmp->icmp_type != ICMP_UNREACH && !(sess->protocol==2 && icmp->icmp_type == ICMP_TIMXCEED) && !(sess->protocol==3 && icmp->icmp_type == ICMP_TRACE))
 			return;
 		if(icmp->icmp_type == ICMP_ECHOREPLY)
@@ -750,7 +748,7 @@ void win_icmp_process(lft_session_params * sess)
 #else
 static void pcap_icmp_process_packet(u_char * user_data, const struct pcap_pkthdr *hdr, const u_char * packet)
 {
-    lft_session_params * sess=(lft_session_params *)user_data;
+    lft_session_params * sess=(lft_session_params *)(void *)user_data;
 	if(sess->exit_state<0)
 		return;
     icmp_process_packet(sess, LFTErrHandler, LFTEvtHandler, packet, hdr);
@@ -797,14 +795,30 @@ void icmp_finish(lft_session_params *sess)
     int hopno;
     int maxhop;
     int reply, noreply;
-    int noreply_hop_first = 0;
     int as_for_hop = 0;
     struct trace_packet_info_s *tp;
     char *netname; 
-    char ind;
     /* int ocres; */
     char *myApp = (char *)malloc((strlen(version) * sizeof(char)) + 1 + (strlen(appname) * sizeof(char)));
     struct ip_list_array *ipaslist = (struct ip_list_array *)malloc(sizeof(struct ip_list_array));
+	/* Variables for seam detection */
+	int icmpcode;
+	int asseam_hopno=-1;
+	struct in_addr asseam_hopaddr;
+	int netseam_hopno=-1;
+	struct in_addr netseam_hopaddr;
+	struct in_addr classbmask;
+	struct in_addr masked_target;
+	int prevasn=-1;
+	struct in_addr prevasn_hopaddr;
+	int prevasn_hopno;
+	int lastishole;
+	int netreached=0;
+	int isseam;
+	/* ---------------------------- */
+	inet_aton("255.255.0.0", &classbmask);
+	masked_target.s_addr=sess->remote_address.s_addr & classbmask.s_addr;
+	
     EvtPacketInfoParam ehip;
 
     memset(ipaslist, 0, sizeof(struct ip_list_array));
@@ -924,6 +938,85 @@ void icmp_finish(lft_session_params *sess)
 		free(ipaslist);
 		return;
 	}
+	
+	/* seam detection */
+	as_for_hop=0;
+	for(hopno = sess->ttl_min; hopno < sess->hop_info_length; hopno++)
+	{
+		struct in_addr last_hop;
+		icmpcode=-100;
+		last_hop.s_addr = 0;
+		if(sess->hop_info[hopno].all_rcvd)
+		{
+			lastishole=0;
+            SLIST_FOREACH(tp, &(sess->hop_info[hopno].packets), next_by_hop)
+			{
+                if(tp->recv.tv_sec)
+				{
+					if(hopno<=maxhop)
+						icmpcode=tp->icmp_type;
+					if(last_hop.s_addr != tp->hopaddr.s_addr)
+					{
+						if((tp->hopaddr.s_addr & classbmask.s_addr) == masked_target.s_addr)
+							netreached=1;
+						else
+						{
+							netseam_hopno=hopno;
+							netseam_hopaddr=tp->hopaddr;
+						}
+						if(sess->do_aslookup || sess->do_netlookup)
+						{
+							if (sess->use_radb)
+							{ 
+								/* using RADB/IRR */
+								tp->asnumber = w_lookup_as(sess->wsess, inet_ntoa(tp->hopaddr));
+							}
+							else
+							{
+								/* using pwhois by default */
+								tp->asnumber = (*ipaslist).asn[as_for_hop];
+							}
+							if(prevasn==-1)
+							{
+								if(tp->asnumber)
+								{
+									prevasn=tp->asnumber;
+									prevasn_hopno=hopno;
+									prevasn_hopaddr=tp->hopaddr;
+								}
+							}
+							else
+							{
+								if(tp->asnumber)
+								{
+									if(tp->asnumber!=prevasn)
+									{
+										asseam_hopno=prevasn_hopno;
+										asseam_hopaddr=prevasn_hopaddr;
+									}
+									prevasn=tp->asnumber;
+									prevasn_hopno=hopno;
+									prevasn_hopaddr=tp->hopaddr;
+								}
+							}
+						}
+						last_hop=tp->hopaddr;
+					}
+				}
+			}
+			as_for_hop++;
+		}
+		else
+			lastishole=1;
+		if(icmpcode==-1)
+			break;
+	}
+	if(!netreached)
+		netseam_hopno=-1;
+	if(lastishole)
+		asseam_hopno=-1;
+	/* -------------- */
+
 	noreply = 0;
 	reply = 0;
 	as_for_hop = 0;            /* this correlates the hopno to the asn stored in ipaslist */
@@ -946,7 +1039,6 @@ void icmp_finish(lft_session_params *sess)
 					return;
 				}
 			}
-			noreply_hop_first = 0;
 		}
     
 		last_hop.s_addr = 0;
@@ -988,7 +1080,6 @@ void icmp_finish(lft_session_params *sess)
                                 free(ipaslist);
                                 return;
                         }
-			ind = ' ';
         
 			//printf("hopno=%d all_rcvd=%d",hopno,sess->hop_info[hopno].all_rcvd);
 			SLIST_FOREACH(tp, &(sess->hop_info[hopno].packets), next_by_hop)
@@ -999,7 +1090,6 @@ void icmp_finish(lft_session_params *sess)
                                                                 
 					if (last_hop.s_addr != tp->hopaddr.s_addr)
 					{
-						ind = ' ';
 						ehip.asnumber = 0; 	/* init/clear the ASN */
 						if (sess->do_aslookup)
 						{
@@ -1014,6 +1104,8 @@ void icmp_finish(lft_session_params *sess)
 								ehip.asnumber = (*ipaslist).asn[as_for_hop];
 							}
 						}
+						tp->asnumber=ehip.asnumber;
+						ehip.netname=NULL;
 						if (sess->do_netlookup)
 						{
 							if (!sess->do_aslookup || (sess->do_aslookup && !sess->use_cymru && !sess->use_radb))
@@ -1026,12 +1118,66 @@ void icmp_finish(lft_session_params *sess)
 							}
 							ehip.netname = netname;
 						}
+						if(ehip.netname)
+							strncpy(tp->netname, ehip.netname, 511);
+						else
+							tp->netname[0]=0;
 					}
 					ehip.last_hop = last_hop;
+					tp->last_hop=ehip.last_hop;
 					last_hop = tp->hopaddr;
 				}
 				ehip.tp = tp;
-				ind = '/';
+				/* seam processing */
+				isseam=0;
+				ehip.is_asseam=0;
+				ehip.is_netseam=0;
+				ehip.is_open=0;
+				ehip.is_filtered=0;
+				ehip.seam_traced=0;
+				if(sess->check_seam && hopno==asseam_hopno && tp->hopaddr.s_addr==asseam_hopaddr.s_addr)
+				{
+					isseam=1;
+					ehip.is_asseam=1;
+				}
+				if(sess->check_seam && hopno==netseam_hopno && tp->hopaddr.s_addr==netseam_hopaddr.s_addr)
+				{
+					isseam=1;
+					ehip.is_netseam=1;
+				}
+				if(isseam)
+				{
+					if(sess->check_seam)
+					{
+						int curroutputstyle=getOutputStyle();
+						char hostname[100];
+						ehip.seam_traced=1;
+						setOutputStyle(2);
+						lft_session_params * subsess=LFTSessionOpen();
+						strncpy(hostname, inet_ntoa(tp->hopaddr),100);
+						subsess->senddevsel = 1;
+						subsess->senddev = sess->pcap_send_dev;
+						subsess->userdevsel = 1;
+						subsess->userdev = sess->pcap_dev;
+						subsess->auto_ports=0;
+						subsess->dport=179;
+						subsess->seq_start=30;
+						subsess->retry_min=1;
+						subsess->retry_max=1;
+						subsess->resolve_names=0;
+						subsess->ahead_limit=1;
+						subsess->break_on_icmp = 0;
+						subsess->is_graphviz_subquery=1;
+						subsess->hostname=hostname;
+						subsess->hostname_lsrr_size = 0;
+						LFTExecute(subsess);
+						ehip.is_open=subsess->target_open;
+						ehip.is_filtered=subsess->target_filtered;
+						LFTSessionClose(subsess);
+						setOutputStyle(curroutputstyle);
+					}
+				}
+				/* --------------- */
 				LFTEvtHandler(sess,EVT_RPT_PACKET_INFO,&ehip);
 	                        if(sess->exit_state < 0){
         	                        free(ipaslist);
