@@ -16,6 +16,11 @@
 
 #include "lft_types.h"
 
+/* not available in earlier darwin systems */
+#ifndef AI_NUMERICSERV 
+#define AI_NUMERICSERV 0 
+#endif 
+
 /* As the trace progresses, each hope will attempt
 to work through the states one by one until it
 receives an answer (2 attempts per state).
@@ -32,6 +37,9 @@ following hops to continue from.
 
 #define HF_ENDPOINT         0x01
 
+/* default timeout value */
+#define DEFAULT_TIMEOUT_MS  250 
+
 /* Common EtherType values */
 #ifndef ETHERTYPE_IP
 #define	ETHERTYPE_IP		0x0800	/* IP protocol */
@@ -49,6 +57,24 @@ following hops to continue from.
 #define	ETHERTYPE_IPV6		0x86dd	/* IPv6 */   
 #endif
 
+/* Sometimes-missing BPF values */
+#ifndef DLT_RAW
+#define DLT_RAW         101     /* Raw IP */
+#endif
+#ifndef DLT_PPP_SERIAL
+#define DLT_PPP_SERIAL  50      /* PPP with HDLC encapsulation */
+#endif
+#ifndef DLT_PPP_ETHER
+#define DLT_PPP_ETHER   51      /* PPP over Ethernet */
+#endif
+#ifndef DLT_LINUX_SLL
+#define DLT_LINUX_SLL   113     /* Linux cooked capture */
+#endif
+#ifndef DLT_PPP
+#define DLT_PPP         9       /* PPP over Ethernet */
+#endif
+
+
 /* ToS (type of service) bits we can set on the IP datagram */
 
 #define TOSMINDELAY         0x10
@@ -133,6 +159,11 @@ typedef struct _packetinfoevtparam
     int asnumber;
     const char * netname;
     struct in_addr last_hop;
+	int is_asseam;
+	int is_netseam;
+	int seam_traced;
+	int is_open;
+	int is_filtered;
     const struct trace_packet_info_s * tp;
 }EvtPacketInfoParam;
 #define EVT_RPT_PACKET_INFO             17
@@ -173,7 +204,7 @@ typedef struct _nonseqpack
 typedef struct _recvpacket
 {
     struct in_addr ipaddr;
-    const struct trace_packet_info_s * tp;
+	struct trace_packet_info_s * tp;
     unsigned int seq;
 }EvtRecvPacketParam;
 #define EVT_RECV_PACKET                 41
@@ -209,32 +240,32 @@ typedef struct _incomtcpicmp
 
 #define EVT_DBG_LOG_MESSAGE             57
 
-#define EVT_PROGRESS_SKIP_PACKET		58
+#define EVT_PROGRESS_SKIP_PACKET	58
 
-#define EVT_OPEN_CHECK_RESULT			59
+#define EVT_OPEN_CHECK_RESULT		59
 
-#define ERR_BTCP_PROBE_PORT_IS_BUSY		60
-#define ERR_BTCP_WRONG_PORT_VALUE		61
+#define ERR_BTCP_PROBE_PORT_IS_BUSY	60
+#define ERR_BTCP_WRONG_PORT_VALUE	61
 
-#define EVT_OCHECK_START				62
-#define WRN_OCHECK_OPEN_SOCK			63
-#define WRN_OCHECK_IOCTL				64
-#define WRN_OCHECK_SELECT				65
-#define WRN_OCHECK_GETERROR				66
-#define WRN_OCHECK_SOCKERROR			67
-#define WRN_OCHECK_TIMEOUT				68
-#define EVT_OCHECK_OPEN					69
-#define WRN_OCHECK_FCNTLGET				70
-#define WRN_OCHECK_FCNTLSET				71
-#define WRN_OCHECK_CONNECTERR			72
+#define EVT_OCHECK_START		62
+#define WRN_OCHECK_OPEN_SOCK		63
+#define WRN_OCHECK_IOCTL		64
+#define WRN_OCHECK_SELECT		65
+#define WRN_OCHECK_GETERROR		66
+#define WRN_OCHECK_SOCKERROR		67
+#define WRN_OCHECK_TIMEOUT		68
+#define EVT_OCHECK_OPEN			69
+#define WRN_OCHECK_FCNTLGET		70
+#define WRN_OCHECK_FCNTLSET		71
+#define WRN_OCHECK_CONNECTERR		72
 
 typedef struct _incomechoreplyicmp
 {
     const struct ip * ip;
     const struct icmp_echo_header_s * echo;
 }EvtIncomingICMPEchoParam;
-#define EVT_INCOMING_ICMP_Echo			73
-#define EVT_RCVD_ICMP_Echo				74
+#define EVT_INCOMING_ICMP_Echo		73
+#define EVT_RCVD_ICMP_Echo		74
 
 typedef struct _incomicmpicmp
 {
@@ -243,8 +274,8 @@ typedef struct _incomicmpicmp
 	const struct ip * orig_ip;
     const struct icmp_echo_header_s * echo;
 }EvtIncomingICMPICMPParam;
-#define EVT_INCOMING_ICMP_ICMP			75
-#define EVT_RCVD_ICMP_ICMP				76
+#define EVT_INCOMING_ICMP_ICMP		75
+#define EVT_RCVD_ICMP_ICMP		76
 
 #if defined(BSD_IP_STACK) && !defined(OPENBSD)
 #define SCREWED_IP_LEN
@@ -368,6 +399,8 @@ typedef struct _lft_session_params
     /* hop information, by ttl */
     struct hop_info_s * hop_info;
     const char * pcap_dev;
+    /* data link type as in pcap_datalink() */
+    int pcap_datalink;
     const char * pcap_send_dev;
     const char * userdev;
     const char * senddev;
@@ -376,6 +409,10 @@ typedef struct _lft_session_params
     /*User's data*/
     void * UsersDataCookie;
 
+	/* GraphViz subquery. Disables any output. */
+	int is_graphviz_subquery;
+	int check_seam;
+	char * graphviz_icon_path;
     /*Exit status. When this field has value <0 lft will end work as soon as possible*/
     int exit_state;
 }lft_session_params;
@@ -443,6 +480,8 @@ void lft_printf(lft_session_params * sess, const char *templ, ...);
 /*----------------------------------------------------------------------------*/
 void setOutputStyle(int nstyle); /* 0 - ordinary output, 1 - xml output */
 int outputStyleIsXML(void);
+int outputStyleIsGraphViz(void);
+int getOutputStyle();
 /*----------------------------------------------------------------------------*/
 #ifdef	__cplusplus
 }
