@@ -35,6 +35,7 @@ typedef signed long n_long;
 typedef signed short n_short;
 typedef signed long n_time;
 #include <stdio.h>
+#include <ctype.h>
 #include <math.h>
 #include <time.h>
 #ifdef __CYGWIN__
@@ -98,6 +99,8 @@ typedef signed long n_time;
 #include <sys/ioctl.h>
 #include <net/bpf.h>
 #if !defined(DARWIN) && !defined(NETBSD)
+#define HAVE_SNPRINTF
+#define HAVE_VSNPRINTF
 #include <pcap-int.h>
 #endif
 #endif
@@ -209,6 +212,11 @@ struct trace_packet_info_s
     struct timeval sent;
     struct timeval recv;		/* 0 if unreceived */
     struct in_addr hopaddr;     /* IP address of router */
+	/* copy of EvtPacketInfoParam */
+	int asnumber;
+	char netname[512];
+	struct in_addr last_hop;
+	/*----------------------------*/
 	union
 	{
 	    struct trace_packet_s packet;
