@@ -39,12 +39,15 @@
 
 #include "whois.h"
 
+#if defined(BSD_IP_STACK)
+#define pcap_snprintf    snprintf
+#endif
 
 /*#define ASSERT(x)       if (!(x)) { fprintf(stderr, "Assertion ("#x") failed\n"); exit(EXIT_FAILURE); }*/
 
 /* OPTIONS and variable initialization */
 
-static const char *version = "3.5";             /* set version string for library and client */
+static const char *version = "3.7";             /* set version string for library and client */
 #ifndef STANDALONE
 static const char appname[] = "pWhoIs C Library";   /* set application name */
 #endif
@@ -76,9 +79,20 @@ static int cymrufromfile = 0;                   /* Don't use Cymru by default fo
 static int use_gigo = 1;                        /* Use GIGO feature by default */
 static const unsigned int max_hostname_input = 200;      /* Maximum length of input from user */
 static const int max_lines = 1000;              /* Maximum lines to read from bulk file per query */
+static const int line_size = 256;               /* Maximum line length */
 static char hostname[256];
 #endif
 
+/* Optimize DNS resolution */
+/*
+    #include <resolv.h>
+    extern struct __res_state _res;
+
+    res_init();
+    _res.retrans = 1;
+    _res.retry = 1;
+*/
+
 /* END of OPTIONS and variable initialization */
 #if defined(WIN32) || defined(_WIN32)
 char *
@@ -339,7 +353,7 @@ static char *match_afterparens(char *target)
     else return NULL;
 }
 
-whois_session_params *w_init()
+whois_session_params *w_init(void)
 {
     /* int e; */
     whois_session_params *wsess = (whois_session_params *)malloc(sizeof(whois_session_params));
@@ -419,7 +433,7 @@ w_ask(const char *serv, const char *q, const char *port)
 	setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &whreadtimeout, sizeof(whreadtimeout));
 #endif
 
-    if (connect(s, (struct sockaddr *)&sin4, sizeof (sin4)) < 0) 
+    if (connect(s, (const struct sockaddr *)(const void *)&sin4, sizeof (sin4)) < 0) 
         return NULL;
     
     br_s = 512;
@@ -1165,9 +1179,10 @@ static int w_display_bulkfromfile_pwhois(whois_session_params * wsess, char *fil
     char *serv;
     FILE *bulkFile;
     int num_lines = 0, i = 0;
-    char *lines = malloc(256*max_lines);
-    char *this_line = malloc(256);    
-    char *whob_query = (char *)malloc(((strlen(appname) * sizeof(char) +10) + (strlen(appname_extras) * sizeof(char)) + (strlen(version) * sizeof(char))) + (strlen(query_begin) * sizeof(char)) + (strlen(format_instructions) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (256*max_lines));
+    char *lines = malloc(line_size * max_lines);
+    char *this_line = malloc(line_size);    
+    size_t whob_query_len = ((strlen(appname) * sizeof(char) +10) + (strlen(appname_extras) * sizeof(char)) + (strlen(version) * sizeof(char))) + (strlen(query_begin) * sizeof(char)) + (strlen(format_instructions) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (line_size * max_lines);
+    char *whob_query = (char *)malloc(whob_query_len);
 
     reply = NULL;
     *whob_query = '\0';
@@ -1200,12 +1215,12 @@ static int w_display_bulkfromfile_pwhois(whois_session_params * wsess, char *fil
         if (num_lines >= max_lines && wsess->w_noisy >= 2)
             fprintf(stderr,"Processing next batch of %d beginning at line %d.\n",max_lines,(num_lines+1));
         
-        memset(lines, 0, sizeof(lines));
-        memset(whob_query, 0, sizeof(whob_query)); 
+        memset(lines, 0, line_size * max_lines);
+        memset(whob_query, 0, whob_query_len);
         
         for (i = 0; i < max_lines; i++) {
             
-            if (fgets(this_line,255,bulkFile)) {
+            if (fgets(this_line,line_size - 1,bulkFile)) {
                 if (strncmp(this_line, "#", 1) && strncmp(this_line, ";", 1)) {
                     strcat(lines,this_line);
                     num_lines++;
@@ -1269,9 +1284,10 @@ static int w_display_bulkfromfile_riswhois(whois_session_params * wsess, char *f
     char *serv;
     FILE *bulkFile;
     int num_lines = 0, i = 0;
-    char *lines = (char *)malloc(256*max_lines);
-    char *this_line = (char *)malloc(256);    
-    char *whob_query = (char *)malloc(10 + (strlen(query_begin) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (256*max_lines));
+    char *lines = (char *)malloc(line_size * max_lines);
+    char *this_line = (char *)malloc(line_size);    
+    size_t whob_query_len = 10 + (strlen(query_begin) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (line_size * max_lines);
+    char *whob_query = (char *)malloc(whob_query_len);
 
     reply = NULL;
     *whob_query = '\0';
@@ -1304,12 +1320,12 @@ static int w_display_bulkfromfile_riswhois(whois_session_params * wsess, char *f
         if (num_lines >= max_lines && wsess->w_noisy >= 2)
             fprintf(stderr,"Processing next batch of %d beginning at line %d.\n",max_lines,(num_lines+1));
         
-        memset(lines, 0, sizeof(lines));
-        memset(whob_query, 0, sizeof(whob_query)); 
+        memset(lines, 0, line_size * max_lines);
+        memset(whob_query, 0, whob_query_len);
         
         for (i = 0; i < max_lines; i++) {
             
-            if (fgets(this_line,255,bulkFile)) {
+            if (fgets(this_line,line_size - 1,bulkFile)) {
                 if (strncmp(this_line, "#", 1) && strncmp(this_line, ";", 1)) {
                     strcat(lines,this_line);
                     num_lines++;
@@ -1364,9 +1380,10 @@ static int w_display_bulkfromfile_cymru(whois_session_params * wsess, char *file
     char *serv;
     FILE *bulkFile;
     int num_lines = 0, i = 0;
-    char *lines = (char *)malloc(256*max_lines);
-    char *this_line = (char *)malloc(256);    
-    char *whob_query = (char *)malloc(10 + (strlen(query_begin) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (256*max_lines));
+    char *lines = (char *)malloc(line_size * max_lines);
+    char *this_line = (char *)malloc(line_size);    
+    size_t whob_query_len = 10 + (strlen(query_begin) * sizeof(char)) + (strlen(query_end) * sizeof(char)) + (line_size * max_lines);
+    char *whob_query = (char *)malloc(whob_query_len);
     
     reply = NULL;
     *whob_query = '\0';
@@ -1399,12 +1416,12 @@ static int w_display_bulkfromfile_cymru(whois_session_params * wsess, char *file
         if (num_lines >= max_lines && wsess->w_noisy >= 2)
             fprintf(stderr,"Processing next batch of %d beginning at line %d.\n",max_lines,(num_lines+1));
         
-        memset(lines, 0, sizeof(lines));
-        memset(whob_query, 0, sizeof(whob_query)); 
+        memset(lines, 0, line_size * max_lines);
+        memset(whob_query, 0, whob_query_len);
         
         for (i = 0; i < max_lines; i++) {
             
-            if (fgets(this_line, 255, bulkFile)) {
+            if (fgets(this_line, line_size - 1, bulkFile)) {
                 if (strncmp(this_line, "#", 1) && strncmp(this_line, ";", 1)) {
                     strcat(lines, this_line);
                     num_lines++;
@@ -2296,7 +2313,6 @@ int main(int ac, char **av)
     int user_asn = 0;
     char user_asn_buf[10];
     whois_session_params * wsess;
-    int UseLocalTime=1;
     
     memset(&hostname, 0, sizeof(hostname));
     
@@ -2306,18 +2322,23 @@ int main(int ac, char **av)
     while ((ch = getopt (ac, av, "aCcfgNnOopPRrstuVvh:w:")) != EOF)
         switch (ch) {
             case 'a':
+                use_gigo = 0;
                 show_routes_byasn = 1;
                 break;
             case 'N':
+                use_gigo = 0;
                 show_networks_byasn = 1;
                 break;
             case 'O':
+                use_gigo = 0;
                 show_contacts_byasn = 1;
                 break;
             case 's':
+                use_gigo = 0;
                 show_server_status = 1;
                 break;
             case 'P':
+                use_gigo = 0;
                 show_routes_byprefix = 1;
                 break;
             case 'v':
@@ -2335,7 +2356,6 @@ int main(int ac, char **av)
                 }
 #endif
 #endif
-		UseLocalTime = 0;
                 show_cache_date = 1;
                 break;
             case 't':
@@ -2398,6 +2418,7 @@ int main(int ac, char **av)
                 display_aspath = 1;
                 break;
             case 'f':
+                use_gigo = 0;
                 read_from_file = 1;
                 break;
             case 'V':
@@ -2439,12 +2460,12 @@ int main(int ac, char **av)
         exit(EXIT_SUCCESS);
     }
     
-    if (use_gigo) {
+    if (use_gigo > 0) {
         w_display_pwhois_gigo(wsess, hostname);
         exit(EXIT_SUCCESS);
     }
     
-    if ((show_routes_byasn || show_contacts_byasn || show_networks_byasn) && (strlen(hostname) <= 5) && atoi(hostname)) {
+    if ((show_routes_byasn || show_contacts_byasn || show_networks_byasn) && (strlen(hostname) <= 10) && atoi(hostname)) {
         user_asn = atoi(hostname);
         if (wsess->w_noisy > 1) printf("Using user-supplied ASN %d for lookup.\n",user_asn);
     } else if (show_routes_byprefix == 1) {
@@ -2553,6 +2574,8 @@ int main(int ac, char **av)
         printf("\n");
     }
     w_close(wsess);
+
+    exit(EXIT_SUCCESS);
 }
 #endif
 
